// Copyright 2020 Ammar Yasser. All rights reserved.
// Use of this source code is governed by a GPL v3 license that can be
// found in the LICENSE file.

package com.heda.apk_admin;

import android.app.Activity;
import android.content.Context;

import androidx.annotation.NonNull;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.PluginRegistry.Registrar;

public class ApkAdminPlugin implements FlutterPlugin, ActivityAware {
    private MethodChannel channel;
    private MethodCallHandlerImpl methodCallHandler;

    public static void registerWith(Registrar registrar) {
        ApkAdminPlugin plugin = new ApkAdminPlugin();
        plugin.setupMethodChannel(registrar.messenger(), registrar.context());
        if (registrar.activeContext() instanceof Activity) {
            plugin.startListeningToActivity(registrar.activity());
        }
    }

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding binding) {
        setupMethodChannel(binding.getBinaryMessenger(), binding.getApplicationContext());
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
        channel = null;
        methodCallHandler = null;
    }


    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        startListeningToActivity(binding.getActivity());
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {
        onDetachedFromActivity();
    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
        onAttachedToActivity(binding);
    }

    @Override
    public void onDetachedFromActivity() {
        if (methodCallHandler != null) {
            methodCallHandler._activity = null;
        }
    }

    private void setupMethodChannel(BinaryMessenger messenger, Context context) {
        String CHANNEL_NAME = "com.heda.apk_admin";
        channel = new MethodChannel(messenger, CHANNEL_NAME);
        methodCallHandler = new MethodCallHandlerImpl(context);
        channel.setMethodCallHandler(methodCallHandler);
    }

    private void startListeningToActivity(Activity activity) {
        if (methodCallHandler != null) {
            methodCallHandler._activity = activity;
        }
    }
}
