package com.heda.apk_admin;

import android.app.Activity;
import android.content.Context;

import androidx.annotation.NonNull;

import com.heda.apk_admin.services.ApkBackup;
import com.heda.apk_admin.services.ApkController;
import com.heda.apk_admin.services.ApkExporter;
import com.heda.apk_admin.utils.ChannelError;
import com.heda.apk_admin.utils.AsyncCallback;

import java.util.Map;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

public class MethodCallHandlerImpl implements MethodCallHandler {
    Activity _activity;
    Context _context;

    MethodCallHandlerImpl(Context context) {
        _context = context;
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull final Result result) {
        ApkController apkController = new ApkController(_context);
        ApkBackup apkBackup = new ApkBackup(_context, _activity);

        try {
            switch (call.method) {
                case "getInstalledApps": {
                    boolean includeSystemApps = call.hasArgument("includeSystemApps") && (boolean) call.argument("includeSystemApps");
                    boolean onlyIncludeAppsWithLaunchIntent = call.hasArgument("onlyIncludeAppsWithLaunchIntent") && (boolean) call.argument("onlyIncludeAppsWithLaunchIntent");
                    boolean includeAppIcon = call.hasArgument("includeAppIcon") && (boolean) call.argument("includeAppIcon");
                    apkController.getInstalledApps(includeSystemApps, onlyIncludeAppsWithLaunchIntent, includeAppIcon, new AsyncCallback() {
                        @Override
                        public void onAsyncCallback(final Object installedApps) {
                            if (!_activity.isFinishing()) {
                                _activity.runOnUiThread(new Runnable() {
                                    @Override
                                    public void run() {
                                        result.success(installedApps);
                                    }
                                });
                            }
                        }
                    });

                }
                break;
                case "launchApp": {
                    String packageName = call.argument("packageName");
                    boolean forceAppStore = call.hasArgument("forceAppStore") && (boolean) call.argument("forceAppStore");
                    String openOperationResult = apkController.launchApp(packageName, forceAppStore);
                    result.success(openOperationResult);
                }
                break;
                case "uninstallApp": {
                    String packageName = call.argument("packageName");
                    apkController.uninstallApp(packageName);
                    result.success("OK");

                }
                break;
                case "makeBackup": {
                    String packageName = call.argument("packageName");
                    apkBackup.backupApp(packageName, new AsyncCallback() {
                        @Override
                        public void onAsyncCallback(final Object asyncResult) {
                            if (!_activity.isFinishing()) {
                                _activity.runOnUiThread(new Runnable() {
                                    @Override
                                    public void run() {
                                        result.success(asyncResult);
                                    }
                                });
                            }
                        }
                    });
                }
                break;
                case "getBackups": {
                    Map<String, String> backups = apkBackup.getBackups();
                    result.success(backups);
                }
                break;
                case "installAppFromBackup": {
                    String backupPath = call.argument("apkPath");
                    boolean apkExists = apkBackup.installAppFromBackup(backupPath);
                    result.success(apkExists);
                }
                break;
                case "shareAppViaBluetooth": {
                    String packageName = call.argument("packageName");
                    ApkExporter apkExporter = new ApkExporter(_context, _activity);
                    String shareResult = apkExporter.shareAppViaBluetooth(packageName);
                    result.success(shareResult);
                }
                break;
                default:
                    result.notImplemented();
            }
        } catch (Exception e) {
            ChannelError channelError = handleChannelException(e);
            result.error(channelError.hashCode, channelError.message, channelError.cause);
        }
    }

    private ChannelError handleChannelException(Exception e) {
        ChannelError channelError = new ChannelError();
        channelError.hashCode = String.valueOf(e.hashCode());
        channelError.message = e.getMessage();
        channelError.cause = e.getCause();
        return channelError;
    }
}
