package com.heda.apk_admin.services;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Build;

import androidx.annotation.NonNull;

import com.heda.apk_admin.utils.AsyncCallback;
import com.heda.apk_admin.utils.AsyncHandler;
import com.heda.apk_admin.utils.FilesController;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

public class ApkBackup extends Activity {
    private static final int WRITE_EXTERNAL_REQUEST = 1;

    Context _context;
    Activity _activity;
    AsyncHandler asyncHandler;

    private FilesController filesController;
    private boolean permissionGranted;

    public ApkBackup(Context context, Activity activity) {
        _context = context;
        _activity = activity;
        asyncHandler = new AsyncHandler();
        filesController = new FilesController(_context);
    }

    public void backupApp(final String packageName, final AsyncCallback onFinished) {
        asyncHandler.run(new Runnable() {
            @Override
            public void run() {
                String backupResult;
                boolean externalStoragePermissionGranted = _checkWriteExternalStoragePermission();
                if (externalStoragePermissionGranted) {
                    try {
                        Map<String, Object> appApk = filesController.getAppApk(packageName);
                        File appSource = (File) appApk.get("baseApk");
                        File backupDir = filesController.getBackupsDir();
                        String backupFileName = packageName + "-" + appApk.get("versionName") + ".apk";
                        File backup = new File(backupDir.getPath() + "/" + backupFileName);
                        if (!backup.exists()) {
                            backup.createNewFile();
                        }
                        filesController.copyFile(appSource, backup);
                        backupResult = "BackupCompleted";
                    } catch (Exception ignored) {
                        backupResult = "UnexpectedError";
                    }
                } else {
                    backupResult = "ExternalStoragePermissionRequired";
                }
                if (onFinished != null) {
                    onFinished.onAsyncCallback(backupResult);
                }
            }
        });
        asyncHandler.stop();
    }

    public HashMap<String, String> getBackups() {
        HashMap<String, String> backups = new HashMap<>();
        File backupFolder = filesController.getBackupsDir();
        File[] backupsFiles = backupFolder.listFiles();
        if (backupsFiles != null && backupsFiles.length > 0) {
            for (File backup : backupsFiles) {
                backups.put(backup.getName(), backup.getAbsolutePath());
            }
        }
        return backups;
    }

    public boolean installAppFromBackup(String backupPath) {
        File backup = new File(backupPath);
        Uri backupUri = filesController.getUriFromFile(backup);
        if (backup.exists()) {
            Intent installIntent = new Intent(Intent.ACTION_VIEW);
            installIntent.setDataAndType(backupUri, "application/vnd.android.package-archive");
            installIntent.putExtra(Intent.EXTRA_NOT_UNKNOWN_SOURCE, true);
            installIntent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
            installIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            _context.startActivity(installIntent);
            return true;
        }
        return false;
    }

    private boolean _checkWriteExternalStoragePermission() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (_context.checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE) == PackageManager.PERMISSION_GRANTED) {
                return true;
            }
            _activity.requestPermissions(new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE}, WRITE_EXTERNAL_REQUEST);
            return permissionGranted;
        }
        return true;
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        if (requestCode == WRITE_EXTERNAL_REQUEST) {
            permissionGranted = grantResults[0] == PackageManager.PERMISSION_GRANTED;
        } else {
            super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        }
    }
}
