package com.heda.apk_admin.services;

import android.Manifest;
import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.net.Uri;
import android.os.Build;
import android.util.Log;

import androidx.annotation.NonNull;

import com.heda.apk_admin.utils.FilesController;

import java.io.File;
import java.util.Map;

public class ApkExporter extends Activity {
    private static int BLUETOOTH_REQUEST = 2;

    Context _context;
    Activity _activity;
    boolean permissionGranted;

    private FilesController filesController;

    public ApkExporter(Context context, Activity activity) {
        _activity = activity;
        _context = context;
        filesController = new FilesController(_context);
    }

    public String shareAppViaBluetooth(String packageName) throws NameNotFoundException {
        Map<String, Object> appApk = filesController.getAppApk(packageName);
        File appBaseApk = (File) appApk.get("baseApk");
        Uri apkUri = filesController.getUriFromFile(appBaseApk);
        boolean bluetoothSupported = checkBluetoothAvailability();
        boolean bluetoothPermissionGranted = _checkWBluetoothPermission();
        if (!bluetoothPermissionGranted) {
            return "BluetoothPermissionDenied";
        }
        if (bluetoothSupported) {
            Intent shareApp = new Intent(Intent.ACTION_SEND);
            shareApp.setType("*/*");
            shareApp.setPackage("com.android.bluetooth");
            shareApp.putExtra(Intent.EXTRA_STREAM, apkUri);
            Log.d("com.heda7", "heda7");
            _context.startActivity(Intent.createChooser(shareApp, "Share app").addFlags(Intent.FLAG_ACTIVITY_NEW_TASK));
            return "CalledSuccessfully";
        }
        return "BluetoothNotSupported";
    }

    private boolean checkBluetoothAvailability() {
        BluetoothAdapter bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
        return bluetoothAdapter != null;
    }


    private boolean _checkWBluetoothPermission() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (_context.checkSelfPermission(Manifest.permission.BLUETOOTH) == PackageManager.PERMISSION_GRANTED) {
                return true;
            }
            _activity.requestPermissions(new String[]{Manifest.permission.BLUETOOTH}, BLUETOOTH_REQUEST);
            return permissionGranted;
        }
        return true;
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        if (requestCode == BLUETOOTH_REQUEST) {
            permissionGranted = grantResults[0] == PackageManager.PERMISSION_GRANTED;
        } else {
            super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        }
    }

}
