package com.heda.apk_admin.utils;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.net.Uri;
import android.os.Build;

import androidx.core.content.FileProvider;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Map;

public class FilesController {
    Context _context;

    public FilesController(Context context) {
        _context = context;
    }

    public Map<String, Object> getAppApk(String packageName) throws NameNotFoundException {
        PackageManager packageManager = _context.getPackageManager();
        PackageInfo packageInfo = packageManager.getPackageInfo(packageName, 0);
        File baseApk = new File(packageInfo.applicationInfo.publicSourceDir);
        String versionName = packageInfo.versionName;
        Map<String, Object> result = new HashMap<>();
        result.put("baseApk", baseApk);
        result.put("versionName", versionName);
        return result;
    }

    public File getBackupsDir() {
        String appFilesPath = _context.getExternalFilesDir(null).getAbsolutePath();
        File backupDir = new File(appFilesPath, "backups");
        if (!backupDir.exists()) {
            backupDir.mkdir();
        }
        return backupDir;
    }

    public void copyFile(File source, File destination) throws IOException {
        InputStream inputStream = new FileInputStream(source);
        OutputStream outputStream = new FileOutputStream(destination);
        byte[] buffer = new byte[1024];
        int fileLength;
        while ((fileLength = inputStream.read(buffer)) > 0) {
            outputStream.write(buffer, 0, fileLength);
        }
        inputStream.close();
        outputStream.close();
    }

    public Uri getUriFromFile(File file) {
        Uri fileUri;
        if (Build.VERSION.SDK_INT >= 24) {
            fileUri = FileProvider.getUriForFile(_context, "heda7.provider", file);
        } else {
            fileUri = Uri.fromFile(file);
        }
        return fileUri;
    }
}
