// Copyright 2020 Ammar Yasser. All rights reserved.
// Use of this source code is governed by a GPL v3 license that can be
// found in the LICENSE file.

import 'dart:convert';
import 'dart:typed_data';

class App {
  final String appName, packageName, versionName, installDate, lastUpdateDate;
  final Uint8List decodedIcon;

  App({
    this.appName,
    this.packageName,
    this.versionName,
    this.decodedIcon,
    this.installDate,
    this.lastUpdateDate,
  });

  factory App.fromMap(Map<dynamic, dynamic> map) {
    Uint8List decodedIcon = base64.decode(map["appIcon"] ?? "");
    return App(
      appName: map["appName"],
      packageName: map["packageName"],
      versionName: map["versionName"],
      decodedIcon: decodedIcon,
      installDate: _formatDate(map["installDate"]),
      lastUpdateDate: _formatDate(map["lastUpdateDate"]),
    );
  }

  static String _formatDate(int timestamp) {
    DateTime date = DateTime.fromMillisecondsSinceEpoch(timestamp);
    DateTime local = date.toLocal();
    String formattedDate = "${local.year}/${local.month}/${local.day}";
    return formattedDate;
  }
}
